@@ -21,6 +21,7 @@
 
 #include <jni.h>
 #include <zim/file.h>
+#include <android/log.h>
 #include "org_kiwix_kiwixlib_JNIKiwixReader.h"
 
 #include "common/base64.h"
@@ -33,16 +34,16 @@ JNIEXPORT jlong JNICALL Java_org_kiwix_kiwixlib_JNIKiwixReader_getNativeReader(
 {
   std::string cPath = jni2c(filename, env);
 
+  __android_log_print(ANDROID_LOG_INFO, "kiwix", "Attempting to create reader with: %s", cPath.c_str());
   Lock l;
-  kiwix::Reader* reader = nullptr;
   try {
-    reader = new kiwix::Reader(cPath);
-  } catch (...) {
-    std::cerr << "Unable to load ZIM " << cPath << std::endl;
-    reader = NULL;
+    kiwix::Reader* reader = new kiwix::Reader(cPath);
+    return reinterpret_cast<jlong>(new Handle<kiwix::Reader>(reader));
+  } catch (std::exception& e) {
+    __android_log_print(ANDROID_LOG_WARN, "kiwix", "Error opening ZIM file");
+    __android_log_print(ANDROID_LOG_WARN, "kiwix", e.what());
+    return 0;
   }
-
-  return reinterpret_cast<jlong>(new Handle<kiwix::Reader>(reader));
 }
 
 JNIEXPORT void JNICALL
@@ -62,8 +63,10 @@ Java_org_kiwix_kiwixlib_JNIKiwixReader_getMainPage(JNIEnv* env, jobject obj)
   try {
     std::string cUrl = READER->getMainPage().getPath();
     url = c2jni(cUrl, env);
-  } catch (...) {
-    std::cerr << "Unable to get ZIM main page" << std::endl;
+  } catch (std::exception& e) {
+    __android_log_print(ANDROID_LOG_ERROR, "kiwix", "Unable to get ZIM main page");
+    __android_log_print(ANDROID_LOG_ERROR, "kiwix", e.what());
+    url = NULL;
   }
   return url;
 }
@@ -76,8 +79,10 @@ Java_org_kiwix_kiwixlib_JNIKiwixReader_getId(JNIEnv* env, jobject obj)
   try {
     std::string cId = READER->getId();
     id = c2jni(cId, env);
-  } catch (...) {
-    std::cerr << "Unable to get ZIM id" << std::endl;
+  } catch (std::exception& e) {
+    __android_log_print(ANDROID_LOG_ERROR, "kiwix", "Unable to get ZIM id");
+    __android_log_print(ANDROID_LOG_ERROR, "kiwix", e.what());
+    id = NULL;
   }
 
   return id;
@@ -91,8 +96,9 @@ Java_org_kiwix_kiwixlib_JNIKiwixReader_getFileSize(JNIEnv* env, jobject obj)
   try {
     int cSize = READER->getFileSize();
     size = c2jni(cSize);
-  } catch (...) {
-    std::cerr << "Unable to get ZIM file size" << std::endl;
+  } catch (std::exception& e) {
+    __android_log_print(ANDROID_LOG_ERROR, "kiwix", "Unable to get ZIM file size");
+    __android_log_print(ANDROID_LOG_ERROR, "kiwix", e.what());
   }
 
   return size;
@@ -106,8 +112,10 @@ Java_org_kiwix_kiwixlib_JNIKiwixReader_getCreator(JNIEnv* env, jobject obj)
   try {
     std::string cCreator = READER->getCreator();
     creator = c2jni(cCreator, env);
-  } catch (...) {
-    std::cerr << "Unable to get ZIM creator" << std::endl;
+  } catch (std::exception& e) {
+    __android_log_print(ANDROID_LOG_ERROR, "kiwix", "Unable to get ZIM creator");
+    __android_log_print(ANDROID_LOG_ERROR, "kiwix", e.what());
+    creator = NULL;
   }
 
   return creator;
@@ -121,8 +129,10 @@ Java_org_kiwix_kiwixlib_JNIKiwixReader_getPublisher(JNIEnv* env, jobject obj)
   try {
     std::string cPublisher = READER->getPublisher();
     publisher = c2jni(cPublisher, env);
-  } catch (...) {
-    std::cerr << "Unable to get ZIM creator" << std::endl;
+  } catch (std::exception& e) {
+    __android_log_print(ANDROID_LOG_ERROR, "kiwix", "Unable to get ZIM publish");
+    __android_log_print(ANDROID_LOG_ERROR, "kiwix", e.what());
+    publisher = NULL;
   }
   return publisher;
 }
@@ -135,8 +145,10 @@ Java_org_kiwix_kiwixlib_JNIKiwixReader_getName(JNIEnv* env, jobject obj)
   try {
     std::string cName = READER->getName();
     name = c2jni(cName, env);
-  } catch (...) {
-    std::cerr << "Unable to get ZIM name" << std::endl;
+  } catch (std::exception& e) {
+    __android_log_print(ANDROID_LOG_ERROR, "kiwix", "Unable to get ZIM name");
+    __android_log_print(ANDROID_LOG_ERROR, "kiwix", e.what());
+    name = NULL;
   }
   return name;
 }
@@ -154,8 +166,10 @@ Java_org_kiwix_kiwixlib_JNIKiwixReader_getFavicon(JNIEnv* env, jobject obj)
         base64_encode(reinterpret_cast<const unsigned char*>(cContent.c_str()),
                       cContent.length()),
         env);
-  } catch (...) {
-    std::cerr << "Unable to get ZIM favicon" << std::endl;
+  } catch (std::exception& e) {
+    __android_log_print(ANDROID_LOG_ERROR, "kiwix", "Unable to get ZIM favicon");
+    __android_log_print(ANDROID_LOG_ERROR, "kiwix", e.what());
+    favicon = NULL;
   }
   return favicon;
 }
@@ -168,8 +182,10 @@ Java_org_kiwix_kiwixlib_JNIKiwixReader_getDate(JNIEnv* env, jobject obj)
   try {
     std::string cDate = READER->getDate();
     date = c2jni(cDate, env);
-  } catch (...) {
-    std::cerr << "Unable to get ZIM date" << std::endl;
+  } catch (std::exception& e) {
+    __android_log_print(ANDROID_LOG_ERROR, "kiwix", "Unable to get ZIM date");
+    __android_log_print(ANDROID_LOG_ERROR, "kiwix", e.what());
+    date = NULL;
   }
   return date;
 }
@@ -182,8 +198,10 @@ Java_org_kiwix_kiwixlib_JNIKiwixReader_getLanguage(JNIEnv* env, jobject obj)
   try {
     std::string cLanguage = READER->getLanguage();
     language = c2jni(cLanguage, env);
-  } catch (...) {
-    std::cerr << "Unable to get ZIM language" << std::endl;
+  } catch (std::exception& e) {
+    __android_log_print(ANDROID_LOG_ERROR, "kiwix", "Unable to get ZIM language");
+    __android_log_print(ANDROID_LOG_ERROR, "kiwix", e.what());
+    language = NULL;
   }
 
   return language;
@@ -199,8 +217,10 @@ JNIEXPORT jstring JNICALL Java_org_kiwix_kiwixlib_JNIKiwixReader_getMimeType(
     auto entry = READER->getEntryFromEncodedPath(cUrl);
     auto cMimeType = entry.getMimetype();
     mimeType = c2jni(cMimeType, env);
-  } catch (...) {
-    std::cerr << "Unable to get mime-type for url " << cUrl << std::endl;
+  } catch (std::exception& e) {
+    __android_log_print(ANDROID_LOG_ERROR, "kiwix", "Unable to get mime-type for url: %s", cUrl.c_str());
+    __android_log_print(ANDROID_LOG_ERROR, "kiwix", e.what());
+    mimeType = NULL;
   }
   return mimeType;
 }
@@ -230,8 +250,9 @@ JNIEXPORT jbyteArray JNICALL Java_org_kiwix_kiwixlib_JNIKiwixReader_getContent(
 
     setStringObjValue(entry.getMimetype(), mimeTypeObj, env);
     setStringObjValue(entry.getTitle(), titleObj, env);
-  } catch (...) {
-    std::cerr << "Unable to get content for url " << cUrl << std::endl;
+  } catch (std::exception& e) {
+    __android_log_print(ANDROID_LOG_ERROR, "kiwix", "Unable to get content for url: %s", cUrl.c_str());
+    __android_log_print(ANDROID_LOG_ERROR, "kiwix", e.what());
   }
 
   return data;
@@ -261,9 +282,9 @@ JNIEXPORT jbyteArray JNICALL Java_org_kiwix_kiwixlib_JNIKiwixReader_getContentPa
           data, 0, cLen, reinterpret_cast<const jbyte*>(blob.data()));
       setIntObjValue(cLen, sizeObj, env);
     }
-  } catch (...) {
-     std::cerr << "Unable to get partial content for url " << cUrl
-               << "(" << cOffset << ":" << cLen << ")" << std::endl;
+  } catch (std::exception& e) {
+    __android_log_print(ANDROID_LOG_ERROR, "kiwix", "Unable to get partial content for url: %s (%u : %u)", cUrl.c_str(), cOffset, cLen);
+    __android_log_print(ANDROID_LOG_ERROR, "kiwix", e.what());
   }
   return data;
 }
@@ -283,10 +304,9 @@ Java_org_kiwix_kiwixlib_JNIKiwixReader_getDirectAccessInformation(
     entry = entry.getFinalEntry();
     auto part_info = entry.getDirectAccessInfo();
     setPairObjValue(part_info.first, part_info.second, pair, env);
-  } catch (...) {
-    std::cerr << "Unable to locate direct access information for url " << cUrl
-              << std::endl;
-
+  } catch (std::exception& e) {
+    __android_log_print(ANDROID_LOG_ERROR, "kiwix", "Unable to get direct access info for url: %s", cUrl.c_str());
+    __android_log_print(ANDROID_LOG_ERROR, "kiwix", e.what());
   }
   return pair;
 }
@@ -305,9 +325,9 @@ Java_org_kiwix_kiwixlib_JNIKiwixReader_searchSuggestions(JNIEnv* env,
     if (READER->searchSuggestionsSmart(cPrefix, cCount)) {
       retVal = JNI_TRUE;
     }
-  } catch (...) {
-    std::cerr << "Unable to search suggestions for pattern " << cPrefix
-              << std::endl;
+  } catch (std::exception& e) {
+    __android_log_print(ANDROID_LOG_WARN, "kiwix", "Unable to get search results for pattern: %s", cPrefix.c_str());
+    __android_log_print(ANDROID_LOG_WARN, "kiwix", e.what());
   }
 
   return retVal;
@@ -326,8 +346,9 @@ Java_org_kiwix_kiwixlib_JNIKiwixReader_getNextSuggestion(JNIEnv* env,
       setStringObjValue(cTitle, titleObj, env);
       retVal = JNI_TRUE;
     }
-  } catch (...) {
-    std::cerr << "Unable to get next suggestion" << std::endl;
+  } catch (std::exception& e) {
+    __android_log_print(ANDROID_LOG_WARN, "kiwix", "Unable to get next suggestion");
+    __android_log_print(ANDROID_LOG_WARN, "kiwix", e.what());
   }
 
   return retVal;
@@ -346,8 +367,9 @@ Java_org_kiwix_kiwixlib_JNIKiwixReader_getPageUrlFromTitle(JNIEnv* env,
     entry = entry.getFinalEntry();
     setStringObjValue(entry.getPath(), urlObj, env);
     return JNI_TRUE;
-  } catch (...) {
-    std::cerr << "Unable to get URL for title " << cTitle << std::endl;
+  } catch (std::exception& e) {
+    __android_log_print(ANDROID_LOG_WARN, "kiwix", "Unable to get url for title %s: ", cTitle.c_str());
+    __android_log_print(ANDROID_LOG_WARN, "kiwix", e.what());
   }
 
   return JNI_FALSE;
@@ -361,8 +383,10 @@ JNIEXPORT jstring JNICALL Java_org_kiwix_kiwixlib_JNIKiwixReader_getTitle(
   try {
     std::string cTitle = READER->getTitle();
     title = c2jni(cTitle, env);
-  } catch (...) {
-    std::cerr << "Unable to get ZIM title" << std::endl;
+  } catch (std::exception& e) {
+    __android_log_print(ANDROID_LOG_ERROR, "kiwix", "Unable to get zim title");
+    __android_log_print(ANDROID_LOG_ERROR, "kiwix", e.what());
+    title = NULL;
   }
   return title;
 }
@@ -375,8 +399,10 @@ Java_org_kiwix_kiwixlib_JNIKiwixReader_getDescription(JNIEnv* env, jobject obj)
   try {
     std::string cDescription = READER->getDescription();
     description = c2jni(cDescription, env);
-  } catch (...) {
-    std::cerr << "Unable to get ZIM description" << std::endl;
+  } catch (std::exception& e) {
+    __android_log_print(ANDROID_LOG_ERROR, "kiwix", "Unable to get zim description");
+    __android_log_print(ANDROID_LOG_ERROR, "kiwix", e.what());
+    description = NULL;
   }
   return description;
 }
@@ -391,9 +417,9 @@ JNIEXPORT jboolean JNICALL Java_org_kiwix_kiwixlib_JNIKiwixReader_getRandomPage(
     std::string cUrl = READER->getRandomPage().getPath();
     setStringObjValue(cUrl, urlObj, env);
     retVal = JNI_TRUE;
-  } catch (...) {
-    std::cerr << "Unable to get random page" << std::endl;
+  } catch (std::exception& e) {
+    __android_log_print(ANDROID_LOG_ERROR, "kiwix", "Unable to get random page");
+    __android_log_print(ANDROID_LOG_ERROR, "kiwix", e.what());
   }
-
   return retVal;
 }
